#include <mega8.h>
#include <delay.h>
#include <math.h>


const float P2=6.283185308;
const float DR=0.017453293;
const float K1=0.262516090;

const unsigned char days_month[12]={31,28,31,30,31,30,31,31,30,31,30,31};
unsigned char  hr=0, mins=0, sec, day=1, B5=69, C0, H=3, lat=33, cor=30, M=3; //

long int J3, Y=2009;

long int MM[2], HH[2];

float aA[2], aD[2];

float D, T, T0, TT, A, B, A0, D0, D1, E, F, J, S, L, G, V, U, V0, V1, V2; 

bit M8, W8, st, change=0;

unsigned char time_mart, time_octb;


// Declare your global variables here

float Abs(float x)
{
  return(x<0?-x:x);
}


int Sgn(float x)
{  
  return(x>=0?1:-1);
}

long int Trunc (float x)
{
  if (x>=0) {x=floor(x);}
  else {x=ceil(x);}
  return (long int)x;
}


void Calendar(void)
{
  D1 = Trunc(D);
  F = D - D1 - 0.5; 
  J = -Trunc(1.75 * (Trunc((M + 9.0) / 12) + Y));                                      
  S = Sgn(M - 9);
  J3 = Trunc(Y + S * Trunc(Abs(M - 9)/7));
  J3 =( -(((J3 / 100) + 1) * 0.75));
  J = J + Trunc(30.555555556 * M ) + D1 + J3;
  J = J + 1721029 + 367 * Y;
  if (F >= 0) {return;};
  F = F + 1;
  J = J - 1;
}

void GetTimeZone(void)
{
  S = 24110.5 + 236.555367912 * T;
  S = S -(3609.858333333 * (float) H) + 240.0 * lat;
  S = S / 86400;
  S = S - Trunc(S);
  T0 = S * 360 * DR;
} 

float GetAtan()
{
  return atan(S / sqrt(1 - S * S));
}


void PosOfSun(char ind)
{

  //       
  //  (Van Flandern & Pulkkinen, 1979)
  L = 0.779072 + 0.00273790931 * T;
  G = 0.993126 + 0.0027377785 * T;
  L = L - Trunc(L);
  G = G - Trunc(G);
  L = L * P2;
  G = G * P2;
  V = 0.39785 * sin(L);
  V = V - 0.01000 * sin(L - G);
  V = V + 0.00333 * sin(L + G);
  V = V - 0.00021 * TT * sin(L);   
  U = 1 - 0.03349 * cos(G);
  U = U - 0.00014 * cos(2 * L);
  U = U + 0.00008 * cos(L);      
  S = -0.00010 - 0.04129 * sin(2 * L);
  S = S + 0.03211 * sin(G);
  S = S + 0.00104 * sin(2 * L - G);
  S = S - 0.00035 * sin(2 * L + G);
  S = S - 0.00008 * TT * sin(G);
  S = S / sqrt(U - V * V);
  aA[ind] = L + GetAtan();
  S = V / sqrt(U);
  aD[ind] = GetAtan();
}

float getV(float vd, float vh)
{
  return (S * sin(vd) + cos(B5 * DR)* cos(vd) * cos(vh) - cos(DR * 90.833));
}

void PossibleEvents(int Hour)
{
  L = aA[0] + ((float)Hour + 1) / 24 * (aA[1] - aA[0]);
  G = aD[0] + ((float)Hour + 1) / 24 * (aD[1] - aD[0]);
  st = 0;
  A = T0 + Hour * K1 - A0;
  B = T0 + Hour * K1 + K1 - L;
  if (Hour <= 0) {V0 = getV(D0, A);}
  V2 = getV(G,B);
  if (Sgn(V0) == Sgn(V2)) {goto end2;}
  V1 = getV((G + D0) / 2,(B + A) / 2);
  A = 2 * V2 - 4 * V1 + 2 * V0;
  B = 4 * V1 - 3 * V0 - V2;
  D = B * B - 4 * A * V0;
  if (D < 0) {goto end2;};
  D = sqrt(D);
  E = (-B + D) / (2 * A);
  if ((E > 1) || (E < 0)) {E = (-B - D) / (2 * A);} 
  A = (Hour + E + 1 / 120 );
  B = Trunc((A - Trunc(A)) * 60); //M3
  if ((V0 < 0) && (V2 > 0)) {HH[0]=Trunc(A);  MM[0]=Trunc(B);  M8 = 1; st=1;};
  if ((V0 > 0) && (V2 < 0)) {HH[1]=Trunc(A);  MM[1]=Trunc(B);  W8 = 1; st=1;};
end2:   
  A0 = L;
  D0 = G;
  V0 = V2;
}


void OutInform(void)
{
  if ((M8 == 0) && (W8 == 0))
 {  
    if (V2 < 0) {st=1;}//   
    if (V2 > 0) {st=0;}//   
 }
  else
  {
    if (M8 == 0) {st=1;} //     
    if (W8 == 0) {st=0;} //     
  }
}

char Get_Day(int _day, int _month)//   ,   0- 1- ... 
{ 
  int a, y, m; 
  a = (14 - _month) / 12; 
  y = Y - a; 
  m = _month + 12 * a - 2; 
  return (7000 + (_day + y + y / 4 - y / 100 + y / 400 + (31 * m) / 12)) % 7; 
};

void show_info(void)
{ 
  D = day;
  Calendar();
  T = (J - 2451545) + F;
  TT = T / 36525 + 1; // TT - ,   1900
  GetTimeZone(); //   
  T = T - (float) H / 24;
  PosOfSun(0); //   
  T = T + 1;
  PosOfSun(1); 
  if (aA[1] < aA[0]) {aA[1] = aA[1] + P2;}
  S = sin(B5 * DR);
  M8 = 0;
  W8 = 0;
  A0 = aA[0];
  D0 = aD[0];
  for (C0 = 0; C0 < 24; C0++) {PossibleEvents(C0);}
  OutInform();
  for (C0 = 25; C0 < 32; C0++)
  {
    if (Get_Day(C0,3)==0) {time_mart=C0;};
  }
  for (C0 = 25; C0 < 32; C0++)
  {
    if (Get_Day(C0,10)==0) {time_octb=C0;};
  }
}

